var ViewFloatMask = function(maskEngine) {
    var keypressHandler = function (e, element, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var thousandsCode = opt.thousands.charCodeAt();
        var isThousands = (thousandsCode === e.which);
        var is_number = (e.which >= 48 && e.which <= 57);
        var is_dot = (e.which == 46);
        var is_comma = (e.which == 44);
        var is_minus = (e.which == 45);
        var hasSeparator = (element.val().indexOf(opt.decimal) != -1 || element.val().indexOf(opt.thousands) != -1);
        var hasNegative = (element.val().length > 0 || !opt.allowNegative);

        if ((hasSeparator && (is_dot || is_comma)) || (hasNegative && is_minus) || isThousands)
            e.preventDefault();

        if (is_dot || is_comma || is_number || is_minus)
            element.data('maskOptions', opt);
        else
            e.preventDefault();
    };

    var viewFloat = function(element, viewValue, params, callBack) {
        var value = viewValue || '';
        element.val(value);
        params.callback = callBack;
        params.showRound = !('showRound' in params) ? true : params.showRound;
        params.showSymbol = false;

        element.on("keypress", function(event) {
            keypressHandler(event, element, params);
        });

        element.on("blur", function() {
            element.text(unMaskViewFloat(element.val(), params));
            element.val(viewFloatFormat(element.val(), params));
            params.callback(element.text());
        });

        if (!!params.showRound) {
            element.on("focus", function() {
                var value = element.text();
                var viewValue = viewFloatFormat(element.val(), params);
                var currentValue = viewFloatFormat(element.text(), params);

                if (currentValue !== viewValue)
                    value = element.val();

                element.val(viewFloatFormat(value, params));
                element.select();
            });
        }
    };

    var viewFloatFormat = function(value, params) {
        var defaultOptions = Util.clone($.currency.i18n[$.currency.lang]);

        var valueFormated = "";
        try {
            var options = Util.extend(defaultOptions, (params || {}), true);
            valueFormated = $.currency.string(value, options);
        } catch (e) {
            valueFormated = value;
        }
        return valueFormated;
    };

    var unMaskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);
        var fltValue = opt.allowZero && !opt.allowEmpty ? 0 : "";
        var valueFormated = value;

        if (!Util.isEmptyOrBlank(valueFormated)) {
            var decimal = opt.decimal;
            var thousands = opt.thousands;

            if (typeof valueFormated === "string") {
                if(opt.showSymbol)
                    valueFormated = valueFormated.replace(opt.symbol, '');

                if(thousands)
                    valueFormated = valueFormated.replace(new RegExp('\\' + thousands, 'g'), '');

                valueFormated = valueFormated.replace(new RegExp('\\' + decimal, 'g'), '.');
                valueFormated = removeInvalidChar(valueFormated);
            }
            fltValue = parseFloat(valueFormated);

            if ((!opt.allowZero && fltValue === 0) || $.currency.isInvalidValue(fltValue, opt))
                valueFormated = "";
        }
        return valueFormated;
    };

    var removeInvalidChar = function(value) {
        var index = value.indexOf('.');
        return index + 1 === value.length ? value.slice(0, index) : value;
    };

    var maskViewFloat = function(value, params) {
        var opt = Util.extend(Util.clone($.currency.i18n[$.currency.lang]), (params || {}), true);

        if (value)
            value = value.replace(new RegExp('\\' + '.', 'g'), opt.decimal);

        return value;
    };

    this.register = maskEngine.registerMask('viewFloat', viewFloat, viewFloatFormat);
};

Configuration(function(ContextRegister) {
    ContextRegister.register('ViewFloatMask', ViewFloatMask);
});